@@ -1,7 +1,7 @@
 /* -*-mode: c; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-*/
 
 /*
-** Copyright (C) 2008-2013 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2008-2016 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -14,7 +14,7 @@
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software Foundation,
+** along with this program; if not, write to 59the Free Software Foundation,
 ** Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 **
 */
@@ -27,6 +27,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <stdlib.h>
 
 #include <string.h>
 #include <errno.h>
@@ -116,7 +117,7 @@ mu_maildir_mkdir (const char* path, mode_t mode, gboolean noindex, GError **err)
 {
 	g_return_val_if_fail (path, FALSE);
 
-	MU_WRITE_LOG ("%s (%s, %o, %s)", __FUNCTION__,
+	MU_WRITE_LOG ("%s (%s, %o, %s)", __func__,
 		      path, mode, noindex ? "TRUE" : "FALSE");
 
 	if (!create_maildir (path, mode, err))
@@ -162,10 +163,11 @@ get_target_fullpath (const char* src, const gchar *targetpath, GError **err)
 
 	srcfile = g_path_get_basename (src);
 
-	/* create targetpath; note: make the filename cough* unique by
-	 *including a hash * of the srcname in the targetname. This
-	 *helps if there are * copies of a message (which all have the
-	 *same basename)*/
+	/* create targetpath; note: make the filename *cough* unique
+	 * by including a hash of the srcname in the targetname. This
+	 * helps if there are copies of a message (which all have the
+	 * same basename)
+	 */
 	targetfullpath = g_strdup_printf ("%s%c%s%c%u_%s",
 					  targetpath,
 					  G_DIR_SEPARATOR,
@@ -236,7 +238,7 @@ process_file (const char* fullpath, const gchar* mdir,
 		g_debug ("callback said 'MU_STOP' for %s", fullpath);
 	else if (result == MU_ERROR)
 		g_warning ("%s: error in callback (%s)",
-			   __FUNCTION__, fullpath);
+			   __func__, fullpath);
 
 	return result;
 }
@@ -246,21 +248,20 @@ process_file (const char* fullpath, const gchar* mdir,
  * determine if path is a maildir leaf-dir; ie. if it's 'cur' or 'new'
  * (we're skipping 'tmp' for obvious reasons)
  */
-G_GNUC_CONST static gboolean
-is_maildir_new_or_cur (const char *path)
+gboolean
+mu_maildir_is_leaf_dir (const char *path)
 {
 	size_t len;
 
-	g_return_val_if_fail (path, FALSE);
-
 	/* path is the full path; it cannot possibly be shorter
 	 * than 4 for a maildir (/cur or /new) */
-	len = strlen (path);
+	len = path ? strlen (path) : 0;
 	if (G_UNLIKELY(len < 4))
 		return FALSE;
 
-	/* optimization; one further idea would be cast the 4 bytes to an integer
-	 * and compare that -- need to think about alignment, endianness */
+	/* optimization; one further idea would be cast the 4 bytes to an
+	 * integer and compare that -- need to think about alignment,
+	 * endianness */
 
 	if (path[len - 4] == G_DIR_SEPARATOR &&
 	    path[len - 3] == 'c' &&
@@ -329,6 +330,8 @@ ignore_dir_entry (struct dirent *entry, unsigned char d_type)
 {
 	if (G_LIKELY(d_type == DT_REG)) {
 
+		guint u;
+
 		/* ignore emacs tempfiles */
 		if (entry->d_name[0] == '#')
 			return TRUE;
@@ -343,7 +346,18 @@ ignore_dir_entry (struct dirent *entry, unsigned char d_type)
 		if (entry->d_name[0] == 'c' &&
 		    strncmp (entry->d_name, "core", 4) == 0)
 			return TRUE;
-
+		/* ignore tmp/backup files; find the last char */
+		for (u = 0; entry->d_name[u] != '\0'; ++u) {
+			switch (entry->d_name[u]) {
+			case '#':
+			case '~':
+				/* looks like a backup / tempsave file */
+				if (entry->d_name[u + 1] == '\0')
+					return TRUE;
+			default:
+				continue;
+			}
+		}
 		return FALSE; /* other files: don't ignore */
 
 	} else if (d_type == DT_DIR)
@@ -393,12 +407,14 @@ process_dir_entry (const char* path, const char* mdir, struct dirent *entry,
 	d_type = GET_DTYPE(entry, fullpath);
 
 	/* ignore special files/dirs */
-	if (ignore_dir_entry (entry, d_type))
+	if (ignore_dir_entry (entry, d_type)) {
+		/* g_debug ("ignoring %s\n", entry->d_name); */
 		return MU_OK;
+	}
 
 	switch (d_type) {
 	case DT_REG: /* we only want files in cur/ and new/ */
-		if (!is_maildir_new_or_cur (path))
+		if (!mu_maildir_is_leaf_dir (path))
 			return MU_OK;
 
 		return process_file (fullpath, mdir, cb_msg, data);
@@ -505,7 +521,7 @@ process_dir (const char* path, const char* mdir,
 	     gboolean full, void *data)
 {
 	MuError result;
-	DIR* dir;
+	DIR*	dir;
 
 	/* if it has a noindex file, we ignore this dir */
 	if (dir_contains_file (path, MU_MAILDIR_NOINDEX_FILE) ||
@@ -514,27 +530,30 @@ process_dir (const char* path, const char* mdir,
 		return MU_OK;
 	}
 
+	if (dir_cb) {
+		MuError rv;
+		rv = dir_cb (path, TRUE/*enter*/, data);
+		/* ignore this dir; not necessarily an _error_, dir might
+		 * be up-to-date and return MU_IGNORE */
+		if (rv == MU_IGNORE)
+			return MU_OK;
+		else if (rv != MU_OK)
+			return rv;
+	}
+
 	dir = opendir (path);
 	if (!dir) {
 		g_warning ("cannot access %s: %s", path, strerror(errno));
 		return MU_OK;
 	}
 
-	if (dir_cb) {
-		MuError rv;
-		rv = dir_cb (path, TRUE, data);
-		if (rv != MU_OK) {
-			closedir (dir);
-			return rv;
-		}
-	}
-
-	result = process_dir_entries (dir, path, mdir, msg_cb, dir_cb, full, data);
+	result = process_dir_entries (dir, path, mdir, msg_cb, dir_cb,
+				      full, data);
 	closedir (dir);
 
 	/* only run dir_cb if it exists and so far, things went ok */
 	if (dir_cb && result == MU_OK)
-		return dir_cb (path, FALSE, data);
+		return dir_cb (path, FALSE/*leave*/, data);
 
 	return result;
 }
@@ -573,20 +592,22 @@ clear_links (const gchar* dirname, DIR *dir, GError **err)
 	errno = 0;
 	while ((entry = readdir (dir))) {
 
-		const char *fp;
-		char *fullpath;
-		unsigned char d_type;
+		const char	*fp;
+		char		*fullpath;
+		unsigned char	 d_type;
+		guint		 len;
 
 		/* ignore empty, dot thingies */
-		if (!entry->d_name || entry->d_name[0] == '.')
+		if (entry->d_name[0] == '\0' || entry->d_name[0] == '.')
 			continue;
 
 		/* we have to copy the buffer from fullpath_s, because
 		 * it returns a static buffer and we are
 		 * recursive*/
-		fp = mu_str_fullpath_s (dirname, entry->d_name);
-		fullpath = g_newa (char, strlen(fp) + 1);
-		strcpy (fullpath, fp);
+		fp	 = mu_str_fullpath_s (dirname, entry->d_name);
+		len	 = strlen(fp);
+		fullpath = g_newa (char, len + 1);
+		strncpy (fullpath, fp, len);
 
 		d_type = GET_DTYPE (entry, fullpath);
 
@@ -656,6 +677,8 @@ mu_maildir_get_flags_from_path (const char *path)
 		dir  = g_path_get_dirname (path);
 		dir2 = g_path_get_basename (dir);
 
+		flags = MU_FLAG_NONE;
+
 		if (g_strcmp0 (dir2, "new") == 0)
 			flags = MU_FLAG_NEW;
 
@@ -697,8 +720,8 @@ mu_maildir_get_flags_from_path (const char *path)
  * /home/user/Maildir/foo/bar/new/abc  and flags == MU_FLAG_REPLIED
  *    => /home/user/Maildir/foo/bar/cur
  *
- * so the difference is whether MuFlags matches MU_FLAG_NEW is set or
- * not; and in the latter case, no other flags are allowed.
+ * so the difference is whether MU_FLAG_NEW is set or not; and in the
+ * latter case, no other flags are allowed.
  *
  */
 static gchar*
@@ -731,7 +754,7 @@ mu_maildir_get_maildir_from_path (const char* path)
 	if (!g_str_has_suffix (mdir, "cur") &&
 	    !g_str_has_suffix (mdir, "new")) {
 		g_warning ("%s: not a valid maildir path: %s",
-			   __FUNCTION__, path);
+			   __func__, path);
 		g_free (mdir);
 		return NULL;
 	}
@@ -743,12 +766,22 @@ mu_maildir_get_maildir_from_path (const char* path)
 }
 
 
+static char*
+get_new_basename (void)
+{
+	return g_strdup_printf ("%u.%08x%08x.%s",
+				(guint)time(NULL),
+				g_random_int(),
+				(gint32)g_get_monotonic_time (),
+				g_get_host_name ());
+}
+
 
 char*
 mu_maildir_get_new_path (const char *oldpath, const char *new_mdir,
-			 MuFlags newflags)
+			 MuFlags newflags, gboolean new_name)
 {
-	char *mfile, *mdir, *custom_flags, *newpath, *cur;
+	char *mfile, *mdir, *custom_flags, *newpath;
 
 	g_return_val_if_fail (oldpath, NULL);
 
@@ -759,16 +792,22 @@ mu_maildir_get_new_path (const char *oldpath, const char *new_mdir,
 	if (!mdir)
 		return NULL;
 
-	/* determine the name of the mailfile, stripped of its flags, as well
-	 * as any custom (non-standard) flags */
-	mfile = g_path_get_basename (oldpath);
-	for (cur = &mfile[strlen(mfile)-1]; cur > mfile; --cur) {
-		if ((*cur == ':' || *cur == '!') &&
-		    (cur[1] == '2' && cur[2] == ',')) {
-			/* get the custom flags (if any) */
-			custom_flags = mu_flags_custom_from_str (cur + 3);
-			cur[0] = '\0'; /* strip the flags */
-			break;
+	if (new_name)
+		mfile = get_new_basename ();
+	else {
+		/* determine the name of the mailfile, stripped of its flags, as
+		 * well as any custom (non-standard) flags */
+		char *cur;
+		mfile = g_path_get_basename (oldpath);
+		for (cur = &mfile[strlen(mfile)-1]; cur > mfile; --cur) {
+			if ((*cur == ':' || *cur == '!') &&
+			    (cur[1] == '2' && cur[2] == ',')) {
+				/* get the custom flags (if any) */
+				custom_flags =
+					mu_flags_custom_from_str (cur + 3);
+				cur[0] = '\0'; /* strip the flags */
+				break;
+			}
 		}
 	}
 
@@ -781,9 +820,28 @@ mu_maildir_get_new_path (const char *oldpath, const char *new_mdir,
 	return newpath;
 }
 
+
+static gint64
+get_file_size (const char* path)
+{
+	int		rv;
+	struct stat	statbuf;
+
+	rv = stat (path, &statbuf);
+	if (rv != 0) {
+		/* g_warning ("error: %s", strerror (errno)); */
+		return -1;
+	}
+
+	return (gint64)statbuf.st_size;
+}
+
+
 static gboolean
 msg_move_check_pre (const gchar *src, const gchar *dst, GError **err)
 {
+	gint size1, size2;
+
 	if (!g_path_is_absolute(src))
 		return mu_util_g_set_error
 			(err, MU_ERROR_FILE,
@@ -798,7 +856,16 @@ msg_move_check_pre (const gchar *src, const gchar *dst, GError **err)
 		return mu_util_g_set_error (err, MU_ERROR_FILE,
 					    "cannot read %s",  src);
 
-	if (access (dst, F_OK) == 0)
+	if (access (dst, F_OK) != 0)
+		return TRUE;
+
+	/* target exist; we simply overwrite it, unless target has a different
+	 * size. ignore the exceedingly rare case where have duplicate message
+	 * file names with different content yet the same length. (md5 etc. is a
+	 * bit slow) */
+	size1 = get_file_size (src);
+	size2 = get_file_size (dst);
+	if (size1 != size2)
 		return mu_util_g_set_error (err, MU_ERROR_FILE,
 					    "%s already exists", dst);
 
@@ -837,7 +904,7 @@ msg_move (const char* src, const char *dst, GError **err)
 gchar*
 mu_maildir_move_message (const char* oldpath, const char* targetmdir,
 			 MuFlags newflags, gboolean ignore_dups,
-			 GError **err)
+			 gboolean new_name, GError **err)
 {
 	char *newfullpath;
 	gboolean rv;
@@ -846,7 +913,7 @@ mu_maildir_move_message (const char* oldpath, const char* targetmdir,
 	g_return_val_if_fail (oldpath, FALSE);
 
 	newfullpath = mu_maildir_get_new_path (oldpath, targetmdir,
-					       newflags);
+					       newflags, new_name);
 	if (!newfullpath) {
 		mu_util_g_set_error (err, MU_ERROR_FILE,
 				     "failed to determine targetpath");
