@@ -155,8 +155,8 @@ accumulate_text (MuMsg *msg, MuMsgPart *part, GString **gstrp)
 /* declaration, so we can use it earlier */
 static gboolean
 handle_mime_object (MuMsg *msg, GMimeObject *mobj, GMimeObject *parent,
-                    MuMsgOptions opts, unsigned *index, gboolean decrypted,
-                    MuMsgPartForeachFunc func, gpointer user_data);
+		    MuMsgOptions opts, unsigned *index, gboolean decrypted,
+		    MuMsgPartForeachFunc func, gpointer user_data);
 
 static char*
 get_text_from_mime_msg (MuMsg *msg, GMimeMessage *mmsg, MuMsgOptions opts)
@@ -167,11 +167,11 @@ get_text_from_mime_msg (MuMsg *msg, GMimeMessage *mmsg, MuMsgOptions opts)
 	index = 1;
 	gstr  = g_string_sized_new (4096);
 	handle_mime_object (msg,
-	                    mmsg->mime_part,
+			    mmsg->mime_part,
 			    (GMimeObject *) mmsg,
 			    opts,
 			    &index,
-	                    FALSE,
+			    FALSE,
 			    (MuMsgPartForeachFunc)accumulate_text,
 			    &gstr);
 
@@ -239,15 +239,50 @@ get_part_size (GMimePart *part)
 }
 
 
-static void
+static char*
 cleanup_filename (char *fname)
 {
+	GString *gstr;
 	gchar *cur;
+	gunichar uc;
 
-	/* remove slashes, spaces, colons... */
-	for (cur = fname; *cur; ++cur)
-		if (*cur == '/' || *cur == ' ' || *cur == ':')
-			*cur = '-';
+	gstr = g_string_sized_new (strlen (fname));
+
+	/* replace control characters, slashes, and colons by '-' */
+	for (cur = fname; cur && *cur; cur = g_utf8_next_char (cur)) {
+		uc = g_utf8_get_char (cur);
+		if (g_unichar_iscntrl (uc) || uc == '/' || uc == ':')
+			g_string_append_unichar (gstr, '-');
+		else
+			g_string_append_unichar (gstr, uc);
+	}
+
+	g_free (fname);
+	return g_string_free (gstr, FALSE);
+}
+
+
+/*
+ * when a part doesn't have a filename, it can be useful to 'guess' one based on
+ * its mime-type, which allows other tools to handle them correctly, e.g. from
+ * mu4e.
+ *
+ * For now, we only handle calendar invitations in that way, but others may
+ * follow.
+ */
+static char*
+guess_file_name (GMimeObject *mobj, unsigned index)
+{
+	GMimeContentType *ctype;
+
+	ctype = g_mime_object_get_content_type (mobj);
+
+	/* special case for calendars; map to '.vcs' */
+	if (g_mime_content_type_is_type (ctype, "text", "calendar"))
+		return g_strdup_printf ("vcal-%u.vcs", index);
+
+	/* fallback */
+	return g_strdup_printf ("%u.msgpart", index);
 }
 
 
@@ -279,10 +314,11 @@ mime_part_get_filename (GMimeObject *mobj, unsigned index,
 	}
 
 	if (!fname)
-		fname =	g_strdup_printf ("%u.part", index);
+		fname = guess_file_name (mobj, index);
+
+	/* replace control characters, slashes, and colons */
+	fname = cleanup_filename (fname);
 
-	/* remove slashes, spaces, colons... */
-	cleanup_filename (fname);
 	return fname;
 }
 
@@ -297,6 +333,15 @@ mu_msg_part_get_filename (MuMsgPart *mpart, gboolean construct_if_needed)
 				       mpart->index, construct_if_needed);
 }
 
+const gchar*
+mu_msg_part_get_content_id (MuMsgPart *mpart)
+{
+	g_return_val_if_fail (mpart, NULL);
+	g_return_val_if_fail (GMIME_IS_OBJECT(mpart->data), NULL);
+	return g_mime_object_get_content_id((GMimeObject*)mpart->data);
+}
+
+
 
 static MuMsgPartType
 get_disposition (GMimeObject *mobj)
@@ -387,7 +432,7 @@ handle_encrypted_part (MuMsg *msg, GMimeMultipartEncrypted *part,
 
 	if (dec) {
 		rv = handle_mime_object (msg, dec, (GMimeObject *) part,
-		                         opts, index, TRUE, func, user_data);
+					 opts, index, TRUE, func, user_data);
 		g_object_unref (dec);
 	} else {
 		/* On failure to decrypt, list the encrypted part as
@@ -401,7 +446,7 @@ handle_encrypted_part (MuMsg *msg, GMimeMultipartEncrypted *part,
 		g_return_val_if_fail (GMIME_IS_PART(encrypted), FALSE);
 
 		rv = handle_mime_object (msg, encrypted, (GMimeObject *) part,
-		                         opts, index, FALSE, func, user_data);
+					 opts, index, FALSE, func, user_data);
 	}
 
 	return rv;
@@ -426,7 +471,7 @@ handle_part (MuMsg *msg, GMimePart *part, GMimeObject *parent,
 	if (decrypted)
 		msgpart.part_type |= MU_MSG_PART_TYPE_DECRYPTED;
 	else if ((opts & MU_MSG_OPTION_DECRYPT) &&
-	         GMIME_IS_MULTIPART_ENCRYPTED (parent))
+		 GMIME_IS_MULTIPART_ENCRYPTED (parent))
 		msgpart.part_type |= MU_MSG_PART_TYPE_ENCRYPTED;
 
 
@@ -494,13 +539,13 @@ handle_message_part (MuMsg *msg, GMimeMessagePart *mimemsgpart,
 		mmsg = g_mime_message_part_get_message (mimemsgpart);
 		if (mmsg)
 			return handle_mime_object (msg,
-			                           mmsg->mime_part,
-			                           parent,
-			                           opts,
-			                           index,
-			                           decrypted,
-			                           func,
-			                           user_data);
+						   mmsg->mime_part,
+						   parent,
+						   opts,
+						   index,
+						   decrypted,
+						   func,
+						   user_data);
 	}
 
 	return TRUE;
@@ -508,7 +553,7 @@ handle_message_part (MuMsg *msg, GMimeMessagePart *mimemsgpart,
 
 static gboolean
 handle_multipart (MuMsg *msg, GMimeMultipart *mpart, GMimeObject *parent,
-                  MuMsgOptions opts, unsigned *index, gboolean decrypted,
+		  MuMsgOptions opts, unsigned *index, gboolean decrypted,
 		  MuMsgPartForeachFunc func, gpointer user_data)
 {
 	gboolean res;
@@ -519,8 +564,8 @@ handle_multipart (MuMsg *msg, GMimeMultipart *mpart, GMimeObject *parent,
 	for (i = 0; i < mpart->children->len; i++) {
 		part = (GMimeObject *) mpart->children->pdata[i];
 		res &= handle_mime_object (msg, part, parent,
-		                           opts, index, decrypted,
-		                           func, user_data);
+					   opts, index, decrypted,
+					   func, user_data);
 	}
 
 	return res;
@@ -541,14 +586,14 @@ handle_mime_object (MuMsg *msg, GMimeObject *mobj, GMimeObject *parent,
 			(msg, GMIME_MESSAGE_PART(mobj),
 			 parent, opts, index, decrypted, func, user_data);
 	else if ((opts & MU_MSG_OPTION_VERIFY) &&
-	         GMIME_IS_MULTIPART_SIGNED (mobj)) {
+		 GMIME_IS_MULTIPART_SIGNED (mobj)) {
 		check_signature
 			(msg, GMIME_MULTIPART_SIGNED (mobj), opts);
 		return handle_multipart
 			(msg, GMIME_MULTIPART (mobj), mobj, opts,
 			 index, decrypted, func, user_data);
 	} else if ((opts & MU_MSG_OPTION_DECRYPT) &&
-	           GMIME_IS_MULTIPART_ENCRYPTED (mobj))
+		   GMIME_IS_MULTIPART_ENCRYPTED (mobj))
 		return handle_encrypted_part
 			(msg, GMIME_MULTIPART_ENCRYPTED (mobj),
 			 opts, index, func, user_data);
@@ -573,13 +618,13 @@ mu_msg_part_foreach (MuMsg *msg, MuMsgOptions opts,
 		return FALSE;
 
 	return handle_mime_object (msg,
-	                           msg->_file->_mime_msg->mime_part,
-	                           (GMimeObject *) msg->_file->_mime_msg,
-	                           opts,
-	                           &index,
-	                           FALSE,
-	                           func,
-	                           user_data);
+				   msg->_file->_mime_msg->mime_part,
+				   (GMimeObject *) msg->_file->_mime_msg,
+				   opts,
+				   &index,
+				   FALSE,
+				   func,
+				   user_data);
 }
 
 
@@ -740,7 +785,7 @@ mu_msg_part_get_cache_path (MuMsg *msg, MuMsgOptions opts, guint partid,
 
 	/* g_compute_checksum_for_string may be better, but requires
 	 * rel. new glib (2.16) */
-        dirname = g_strdup_printf ("%s%c%x%c%u",
+	dirname = g_strdup_printf ("%s%c%x%c%u",
 				   mu_util_cache_dir(), G_DIR_SEPARATOR,
 				   g_str_hash (path), G_DIR_SEPARATOR,
 				   partid);
@@ -768,7 +813,7 @@ mu_msg_part_save (MuMsg *msg, MuMsgOptions opts,
 	g_return_val_if_fail (msg, FALSE);
 	g_return_val_if_fail (fullpath, FALSE);
 	g_return_val_if_fail (!((opts & MU_MSG_OPTION_OVERWRITE) &&
-			        (opts & MU_MSG_OPTION_USE_EXISTING)), FALSE);
+				(opts & MU_MSG_OPTION_USE_EXISTING)), FALSE);
 
 	if (!mu_msg_load_msg_file (msg, err))
 		return FALSE;
@@ -780,6 +825,11 @@ mu_msg_part_save (MuMsg *msg, MuMsgOptions opts,
 		part = (GMimeObject*)g_mime_message_part_get_message
 			(GMIME_MESSAGE_PART (part));
 
+	if (!part) {
+		g_set_error (err, MU_ERROR_DOMAIN, MU_ERROR_GMIME,
+			     "part %u does not exist", partidx);
+		return FALSE;
+	}
 	if (!GMIME_IS_PART(part) && !GMIME_IS_MESSAGE(part)) {
 		g_set_error (err, MU_ERROR_DOMAIN, MU_ERROR_GMIME,
 			     "unexpected type %s for part %u",
@@ -838,7 +888,7 @@ mu_msg_find_index_for_cid (MuMsg *msg, MuMsgOptions opts,
 
 	return get_matching_part_index (msg, opts,
 					(MuMsgPartMatchFunc)match_cid,
-					(gpointer)(char*)cid);
+					(gpointer)cid);
 }
 
 struct _RxMatchData {
