@@ -55,12 +55,12 @@ do_wordexp (const char *path)
 	char *dir;
 
 	if (!path) {
-		/* g_debug ("%s: path is empty", __FUNCTION__); */
+		/* g_debug ("%s: path is empty", __func__); */
 		return NULL;
 	}
 
 	if (wordexp (path, &wexp, 0) != 0) {
-		/* g_debug ("%s: expansion failed for %s", __FUNCTION__, path); */
+		/* g_debug ("%s: expansion failed for %s", __func__, path); */
 		return NULL;
 	}
 
@@ -108,7 +108,7 @@ mu_util_dir_expand (const char *path)
 	/* now resolve any symlinks, .. etc. */
 	if (realpath (dir, resolved) == NULL) {
 		/* g_debug ("%s: could not get realpath for '%s': %s", */
-		/* 	 __FUNCTION__, dir, strerror(errno)); */
+		/* 	 __func__, dir, strerror(errno)); */
 		g_free (dir);
 		return NULL;
 	} else
@@ -123,7 +123,7 @@ mu_util_create_tmpdir (void)
 {
 	gchar *dirname;
 
-        dirname =  g_strdup_printf ("%s%cmu-%d%c%x",
+	dirname =  g_strdup_printf ("%s%cmu-%d%c%x",
 				    g_get_tmp_dir(),
 				    G_DIR_SEPARATOR,
 				    getuid(),
@@ -193,45 +193,45 @@ mu_util_check_dir (const gchar* path, gboolean readable, gboolean writeable)
 gchar*
 mu_util_guess_maildir (void)
 {
-        const gchar *mdir1, *home;
+	const gchar *mdir1, *home;
 
-        /* first, try MAILDIR */
-        mdir1 = g_getenv ("MAILDIR");
+	/* first, try MAILDIR */
+	mdir1 = g_getenv ("MAILDIR");
 
-        if (mdir1 && mu_util_check_dir (mdir1, TRUE, FALSE))
-                return g_strdup (mdir1);
+	if (mdir1 && mu_util_check_dir (mdir1, TRUE, FALSE))
+		return g_strdup (mdir1);
 
-        /* then, try <home>/Maildir */
-        home = g_get_home_dir();
-        if (home) {
-                char *mdir2;
-                mdir2 = g_strdup_printf ("%s%cMaildir",
-                        home, G_DIR_SEPARATOR);
-                if (mu_util_check_dir (mdir2, TRUE, FALSE))
-                        return mdir2;
-                g_free (mdir2);
-        }
+	/* then, try <home>/Maildir */
+	home = g_get_home_dir();
+	if (home) {
+		char *mdir2;
+		mdir2 = g_strdup_printf ("%s%cMaildir",
+			home, G_DIR_SEPARATOR);
+		if (mu_util_check_dir (mdir2, TRUE, FALSE))
+			return mdir2;
+		g_free (mdir2);
+	}
 
-        /* nope; nothing found */
-        return NULL;
+	/* nope; nothing found */
+	return NULL;
 }
 
 
 gchar*
 mu_util_guess_mu_homedir (void)
 {
-        const char* home;
+	const char* home;
 
-        /* g_get_home_dir use /etc/passwd, not $HOME; this is better,
-         * as HOME may be wrong when using 'sudo' etc.*/
-        home = g_get_home_dir ();
+	/* g_get_home_dir use /etc/passwd, not $HOME; this is better,
+	 * as HOME may be wrong when using 'sudo' etc.*/
+	home = g_get_home_dir ();
 
-        if (!home) {
-                MU_WRITE_LOG ("failed to determine homedir");
-                return NULL;
-        }
+	if (!home) {
+		MU_WRITE_LOG ("failed to determine homedir");
+		return NULL;
+	}
 
-        return g_strdup_printf ("%s%c%s", home ? home : ".",
+	return g_strdup_printf ("%s%c%s", home ? home : ".",
 				G_DIR_SEPARATOR, ".mu");
 }
 
@@ -437,7 +437,6 @@ mu_util_fputs_encoded (const char *str, FILE *stream)
 	unsigned	 bytes;
 	char		*conv;
 
-	g_return_val_if_fail (str, FALSE);
 	g_return_val_if_fail (stream, FALSE);
 
 	/* g_get_charset return TRUE when the locale is UTF8 */
@@ -557,27 +556,3 @@ mu_util_read_password (const char *prompt)
 
 	return g_strdup (pass);
 }
-
-
-const char*
-mu_util_get_hash (const char* str)
-{
-	unsigned	djbhash, bkdrhash, bkdrseed;
-	unsigned	u;
-	static char	hex[18];
-
-	g_return_val_if_fail (str, NULL);
-
-	djbhash  = 5381;
-	bkdrhash = 0;
-	bkdrseed = 1313;
-
-	for(u = 0; str[u]; ++u) {
-		djbhash  = ((djbhash << 5) + djbhash) + str[u];
-		bkdrhash = bkdrhash * bkdrseed + str[u];
-	}
-
-	snprintf (hex, sizeof(hex), "%08x%08x", djbhash, bkdrhash);
-
-	return hex;
-}
