@@ -1,6 +1,6 @@
 ;;; mu4e-vars.el -- part of mu4e, the mu mail user agent
 ;;
-;; Copyright (C) 2011-2013 Dirk-Jan C. Binnema
+;; Copyright (C) 2011-2016 Dirk-Jan C. Binnema
 
 ;; Author: Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 ;; Maintainer: Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
@@ -57,11 +57,13 @@ link."
 
 (defcustom mu4e-get-mail-command "true"
   "Shell command to run to retrieve new mail.
-Common values are \"offlineimap\", \"fetchmail\" and \"mbsync\",
-but you use arbitrary shell-commands. If you set it to
-\"true\" (the default), the command won't don't anything, which is
-useful if you get your mail without the need to explicitly run any
-scripts, for example when running your own mail-server."
+Common values are \"offlineimap\", \"fetchmail\" or \"mbsync\", but
+arbitrary shell-commands can be used.
+
+When set to the literal string \"true\" (the default), the
+command simply finishes succesfully (running the 'true' command)
+without retrieving any mail. This can be useful when mail is
+already retrieved in another way."
   :type 'string
   :group 'mu4e
   :safe 'stringp)
@@ -81,6 +83,33 @@ scripts, for example when running your own mail-server."
   :group 'mu4e
   :safe 'booleanp)
 
+(defcustom mu4e-index-update-in-background t
+  "Whether to run the automatic mail retrieval in the
+background."
+  :type 'boolean
+  :group 'mu4e
+  :safe 'booleanp)
+
+(defcustom mu4e-index-cleanup t
+  "Whether to run a cleanup face after indexing -- that is, see
+if the is a message in the filesystem for each file in the
+message store. Having this option as `t' ensures that no
+non-existing mesages are shown but can also be quite slow with
+large message stores."
+  :type 'boolean
+  :group 'mu4e
+  :safe 'booleanp)
+
+(defcustom mu4e-index-lazy-check nil
+  "Whether to run do a 'lazy check' for deciding whether to
+indexing a message. When this is set to `t', mu only uses the
+directory timestamps to decide on whether it needs to check the
+messages beneath it, which would miss messages that are modified
+outside mu. On the other hand, it's significantly faster."
+  :type 'boolean
+  :group 'mu4e
+  :safe 'booleanp)
+
 
 (defcustom mu4e-update-interval nil
   "Number of seconds between automatic calls to retrieve mail and
@@ -97,13 +126,15 @@ mu4e."
  You can use this hook for example to `mu4e-get-mail-command' with
  some specific setting.")
 
-(defvar mu4e-hide-index-messages nil
-  "If non-nil, mu4e does not show the \"Indexing...\" messages, or
-  any messages relating to updated contacts.")
+(defcustom mu4e-hide-index-messages nil
+  "Whether to hide the \"Indexing...\" messages, or any messages
+relating to updated contacts."
+  :type 'boolean
+  :group 'mu4e)
 
 (defcustom mu4e-change-filenames-when-moving nil
   "When moving messages to different folders, normally mu/mu4e keep
-the the base filename the same (the flags-part of the filename may
+the base filename the same (the flags-part of the filename may
 change still). With this option set to non-nil, mu4e instead
 changes the filename. This latter behavior works better with some
 IMAP-synchronization programs such as mbsync; the default works
@@ -112,13 +143,11 @@ better with e.g. offlineimap."
   :group 'mu4e
   :safe 'booleanp)
 
-
 (defcustom mu4e-attachment-dir (expand-file-name "~/")
   "Default directory for saving attachments.
 This can be either a string (a file system path), or a function
 that takes a filename and the mime-type as arguments, and returns
-the attachment dir. See Info node `(mu4e) Opening and saving
-attachments' for details."
+the attachment dir. See Info node `(mu4e) Attachments' for details."
   :type 'directory
   :group 'mu4e
   :safe 'stringp)
@@ -126,7 +155,11 @@ attachments' for details."
 (defcustom mu4e-user-mail-address-list `(,user-mail-address)
   "List of e-mail addresses to consider 'my email addresses'.
 I.e. addresses whose presence in an email imply that it is a
-personal message. This is used when indexing messages."
+personal message. Note that the local part (the part before '@')
+of e-mail addresses is case-sensitive, as per RFC531.  In
+practice however, most of the mail systems do not distinguish
+addresses based on the case, so the emails in this list will be
+matched case-insensitively."
   :type '(repeat (string :tag "Address"))
   :group 'mu4e)
 
@@ -138,7 +171,10 @@ personal message. This is used when indexing messages."
   'mu4e-user-mail-address-list "0.9.9.x")
 
 (defcustom mu4e-use-fancy-chars nil
-  "Whether to use fancy (non-ascii) characters."
+  "Whether to use fancy (Unicode) characters for marks and
+threads. You can customize the exact fancy characters used with
+`mu4e-marks' and various `mu4e-headers-..-mark' and
+`mu4e-headers..-prefix' variables."
   :type 'boolean
   :group 'mu4e)
 
@@ -167,6 +203,33 @@ is a shortcut key for the query."
 		   character))
   :group 'mu4e)
 
+(defvar mu4e-bookmarks
+  `( ,(make-mu4e-bookmark
+	:name  "Unread messages"
+	:query "flag:unread AND NOT flag:trashed"
+	:key ?u)
+     ,(make-mu4e-bookmark
+	:name "Today's messages"
+	:query "date:today..now"
+	:key ?t)
+     ,(make-mu4e-bookmark
+	:name "Last 7 days"
+	:query "date:7d..now"
+	:key ?w)
+     ,(make-mu4e-bookmark
+	:name "Messages with images"
+	:query "mime:image/*"
+	:key ?p))
+  "A list of pre-defined queries. Each query is represented by a
+mu4e-bookmark structure with parameters @t{:name} with the name
+of the bookmark, @t{:query} with the query expression (a query
+string or an s-expression that evaluates to query string) and a
+@t{:key}, which is the shortcut-key for the query.
+
+An older form of bookmark, a 3-item list with (QUERY DESCRIPTION
+KEY) is still recognized as well, for backward-compatibility.")
+
+
 (defcustom mu4e-split-view 'horizontal
   "How to show messages / headers.
 A symbol which is either:
@@ -195,8 +258,10 @@ view buffer."
   :type 'boolean
   :group 'mu4e)
 
-(defcustom mu4e-cited-regexp "^ *\\(\\(>+ ?\\)+\\)"
-  "Regular expression that determines whether a line is a citation."
+(defcustom mu4e-cited-regexp  "^\\(\\([[:alpha:]]+\\)\\|\\( *\\)\\)\\(\\(>+ ?\\)+\\)"
+  "Regular expression that determines whether a line is a
+  citation. This recognizes lines starting with numbers of '>'
+  and spaces as well as citations of the type \"John> ... \"."
   :type 'string
   :group 'mu4e)
 
@@ -212,6 +277,32 @@ Suggested possible values are:
   :options '(completing-read ido-completing-read)
   :group 'mu4e)
 
+(defcustom mu4e-context-policy 'ask-if-none
+  "The policy to determine the context when entering the mu4e main view.
+
+If the value is `always-ask', ask the user unconditionally.
+
+In all other cases, if any context matches (using its match
+function), this context is used. Otherwise, if none of the
+contexts match, we have the following choices:
+
+- `pick-first': pick the first of the contexts available (ie. the default)
+- `ask': ask the user
+- `ask-if-none': ask if there is no context yet, otherwise leave it as it is
+-  nil: return nil; leaves the current context as is.
+
+Also see `mu4e-compose-context-policy'."
+  :type '(choice
+	   (const :tag "Always ask what context to use, even if one matches"
+	     'always-ask)
+	   (const :tag "Ask if none of the contexts match" 'ask)
+	   (const :tag "Ask when there's no context yet" 'ask-if-none)
+	   (const :tag "Pick the first context if none match" 'pick-first)
+	   (const :tag "Don't change the context when none match" nil))
+  :safe 'symbolp
+  :group 'mu4e)
+
+
 ;; crypto
 (defgroup mu4e-crypto nil
   "Crypto-related settings."
@@ -277,10 +368,21 @@ their canonical counterpart; useful as an example."
       (list :name name :mail mail)))
 
 (defcustom mu4e-contact-rewrite-function nil
-  "Function to be used for when processing contacts and rewrite
-them, for example you may use this for correcting typo's, changed
-names and adapting addresses or names to company policies. As an
-example of this, see `mu4e-contact-identity'."
+  "Either nil or a function to be used for when processing
+contacts and rewrite them or remove them altogether.
+
+If the function receives the contact as a list of the form
+     (:name NAME :mail EMAIL ... other properties ... )
+(other properties may be there as well)
+
+The function should return either:
+ - nil: remove this contact, or
+- the rewritten cell, or
+- the existing cell as-is
+
+For rewriting, it is recommended to use `plist-put' to set the
+changed parameters, so the other properties stay in place. Those
+are needed for sorting the contacts."
   :type 'function
   :group 'mu4e-compose)
 
@@ -450,17 +552,17 @@ I.e. a message with the draft flag set."
   :group 'mu4e-faces)
 
 (defface mu4e-header-key-face
-  '((t :inherit message-header-name-face :bold t))
+  '((t :inherit message-header-name :bold t))
   "Face for a header key (such as \"Foo\" in \"Subject:\ Foo\")."
   :group 'mu4e-faces)
 
 (defface mu4e-header-value-face
-  '((t :inherit font-lock-doc-face))
+  '((t :inherit font-lock-type-face))
   "Face for a header value (such as \"Re: Hello!\")."
   :group 'mu4e-faces)
 
 (defface mu4e-special-header-value-face
-  '((t :inherit font-lock-variable-name-face))
+  '((t :inherit font-lock-builtin-face))
   "Face for special header values."
   :group 'mu4e-faces)
 
@@ -475,7 +577,7 @@ I.e. a message with the draft flag set."
   :group 'mu4e-faces)
 
 (defface mu4e-highlight-face
-  '((t :inherit font-lock-pseudo-keyword-face :bold t))
+  '((t :inherit highlight))
   "Face for highlighting things."
   :group 'mu4e-faces)
 
@@ -484,9 +586,19 @@ I.e. a message with the draft flag set."
   "Face for a header title in the headers view."
   :group 'mu4e-faces)
 
+(defface mu4e-context-face
+  '((t :inherit mu4e-title-face :bold t))
+  "Face for displaying the context in the modeline."
+  :group 'mu4e-faces)
+
 (defface mu4e-modeline-face
   '((t :inherit font-lock-string-face :bold t))
-  "Face for the query view in the mode-line."
+  "Face for the query in the mode-line."
+  :group 'mu4e-faces)
+
+(defface mu4e-view-body-face
+  '((t :inherit default))
+  "Face for the body in the message-view."
   :group 'mu4e-faces)
 
 (defface mu4e-footer-face
@@ -495,7 +607,7 @@ I.e. a message with the draft flag set."
   :group 'mu4e-faces)
 
 (defface mu4e-url-number-face
-  '((t :inherit font-lock-reference-face :bold t))
+  '((t :inherit font-lock-constant-face :bold t))
   "Face for the number tags for URLs."
   :group 'mu4e-faces)
 
@@ -510,7 +622,7 @@ I.e. a message with the draft flag set."
   :group 'mu4e-faces)
 
 (defface mu4e-cited-2-face
-  '((t :inherit font-lock-type-face :bold nil :italic t))
+  '((t :inherit font-lock-preprocessor-face :bold nil :italic t))
   "Face for cited message parts (level 2)."
   :group 'mu4e-faces)
 
@@ -535,7 +647,7 @@ I.e. a message with the draft flag set."
   :group 'mu4e-faces)
 
 (defface mu4e-cited-7-face
-  '((t :inherit font-lock-preprocessor-face :bold nil :italic t))
+  '((t :inherit font-lock-type-face :bold nil :italic t))
   "Face for cited message parts (level 7)."
   :group 'mu4e-faces)
 
@@ -577,6 +689,7 @@ mu4e-compose-mode."
        ( :name "Attachments"
 	 :shortname "Atts"
 	 :help "Message attachments"
+	 :require-full t
 	 :sortable nil))
      (:bcc .
        ( :name "Bcc"
@@ -637,11 +750,13 @@ mu4e-compose-mode."
        ( :name "Signature"
 	 :shortname "Sgn"
 	 :help "Check for the cryptographic signature"
+	 :require-full t
 	 :sortable nil))
      (:decryption .
        ( :name "Decryption"
 	 :shortname "Dec"
 	 :help "Check the cryptographic decryption status"
+	 :require-full t
 	 :sortable nil))
      (:size .
        ( :name "Size"
@@ -665,8 +780,14 @@ mu4e-compose-mode."
 	 :sortable :subject))
      (:to .
        ( :name "To"
-	 :shortname "T"
+	 :shortname "To"
 	 :help "Recipient of the message"
+	 :sortable t))
+     (:user-agent .
+       ( :name "User-Agent"
+	 :shortname "UA"
+	 :help "Program used for writing this message"
+	 :require-full t
 	 :sortable t)))
   "An alist of all possible header fields and information about them.
 This is used in the user-interface (the column headers in the header list, and
@@ -682,7 +803,12 @@ sort by this field.  This can be either a boolean (nil or t), or a
 symbol for /another/ field. For example, the `:human-date' field
 uses `:date' for that.
 
-Note, `:sortable' does not work for custom header fields.")
+Fields with which have the property `:require-full' set to
+non-nil require a full message; in practice this means that you
+cannot use such fieds as part of `mu4e-headers-fields', but only
+in `mu4e-view-fields.'
+
+Note, `:sortable' is not supported for custom header fields.")
 
 
 (defvar mu4e-header-info-custom
@@ -696,10 +822,10 @@ Note, `:sortable' does not work for custom header fields.")
 	     (+ (length (mu4e-message-field msg :to))
 	       (length (mu4e-message-field msg :cc))))))))
 "A list of custom (user-defined) headers. The format is similar
-to `mu4e-header-info', but addds a :function property, which should
-point to a function that takes a message p-list as argument, and
-returns a string. See the default value of `mu4e-header-info-custom
-for an example.")
+to `mu4e-header-info', but adds a :function property, which
+should point to a function that takes a message p-list as
+argument, and returns a string. See the default value of
+`mu4e-header-info-custom for an example.")
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -726,14 +852,11 @@ for an example.")
 (defvar mu4e~view-headers-buffer nil
   "The headers buffer connected to this view.")
 
-(defvar mu4e~contacts-for-completion nil
-  "List of contacts (ie. 'name <e-mail>').
-This is used by the completion functions in mu4e-compose, filled
-when mu4e starts.")
-
-(defvar mu4e~contact-list nil
-  "List of contacts, where each contact is a plist
-  (:name NAME :mail EMAIL :tstamp TIMESTAMP :freq FREQUENCY).")
+(defvar mu4e~contacts nil
+  "Hash of that maps contacts (ie. 'name <e-mail>') to an integer
+with their sort order. We need to keep this information around to
+quickly re-sort subsets of the contacts in the completions function in
+mu4e-compose.")
 
 (defvar mu4e~server-props nil
   "Properties we receive from the mu4e server process.
@@ -742,7 +865,6 @@ when mu4e starts.")
 (defvar mu4e~headers-last-query nil
   "The present (most recent) query.")
 
-
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; our handlers funcs
