@@ -1,6 +1,6 @@
 ;;; mu4e-view.el -- part of mu4e, the mu mail user agent
 ;;
-;; Copyright (C) 2011-2015 Dirk-Jan C. Binnema
+;; Copyright (C) 2011-2016 Dirk-Jan C. Binnema
 
 ;; Author: Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 ;; Maintainer: Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
@@ -40,6 +40,7 @@
 (require 'epa)
 (require 'epg)
 (require 'thingatpt)
+(require 'calendar)
 
 (eval-when-compile (byte-compile-disable-warning 'cl-functions))
 (require 'cl)
@@ -98,8 +99,9 @@ directory and saves all attachments in the chosen directory."
   :group 'mu4e-view)
 
 (defvar mu4e-view-actions
-  '( ("capture message" . mu4e-action-capture-message)
-     ("view as pdf"     . mu4e-action-view-as-pdf))
+  '( ("capture message"  . mu4e-action-capture-message)
+     ("view as pdf"      . mu4e-action-view-as-pdf)
+     ("show this thread" . mu4e-action-show-thread))
   "List of actions to perform on messages in view mode.
 The actions are of the form:
   (NAME FUNC)
@@ -126,6 +128,13 @@ The first letter of NAME is used as a shortcut character.")
 (defvar mu4e-view-fill-headers t
   "If non-nil, automatically fill the headers when viewing them.")
 
+(defvar mu4e-view-header-field-keymap
+  (let ((map (make-sparse-keymap)))
+    (define-key map [mouse-1] 'mu4e~view-header-field-fold)
+    (define-key map (kbd "TAB") 'mu4e~view-header-field-fold)
+    map)
+  "Keymap used for header fields.")
+
 (defvar mu4e-view-contacts-header-keymap
   (let ((map (make-sparse-keymap)))
     (define-key map [mouse-2] 'mu4e~view-compose-contact)
@@ -150,6 +159,14 @@ The first letter of NAME is used as a shortcut character.")
     map)
   "Keymap used in the \"Attachements\" header field.")
 
+(defcustom mu4e-view-auto-mark-as-read t
+  "Automatically mark messages are 'read' when you read
+them. This is typically the expected behavior, but can be turned
+off, for example when using a read-only file-system."
+  :type 'boolean
+  :group 'mu4e-view)
+
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 
@@ -166,16 +183,14 @@ message extracted at some path.")
   "A mapping of user-visible attachment number to the actual part index.")
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defun mu4e-view-message-with-msgid (msgid)
-  "View message with MSGID.
-This is meant for external programs wanting to show specific
-messages - for example, `mu4e-org'."
-  ;; note: hackish; if mu4e-decryption-policy is non-nil (ie., t or 'ask), we
-  ;; decrpt the message. Since here we don't know if message is encrypted or
-  ;; not, when the policy is 'ask'. we simply assume the user said yes...  the
-  ;; alternative would be to ask for each message, encrypted or not.  maybe we
-  ;; need an extra policy...
-  (mu4e~proc-view msgid mu4e-view-show-images mu4e-decryption-policy))
+(defun mu4e-view-message-with-message-id (msgid)
+  "View message with message-id MSGID. This (re)creates a
+headers-buffer with a search for MSGID, then open a view for that
+message."
+  (mu4e-headers-search (concat "msgid:" msgid) nil nil t msgid t))
+
+(define-obsolete-function-alias 'mu4e-view-message-with-msgid
+  'mu4e-view-message-with-message-id "0.9.17")
 
 (defun mu4e~view-custom-field (msg field)
   "Show some custom header field, or raise an error if it is not
@@ -195,10 +210,12 @@ found."
       (lambda (field)
 	(let ((fieldval (mu4e-message-field msg field)))
 	  (case field
-	    (:subject  (mu4e~view-construct-header field fieldval))
-	    (:path     (mu4e~view-construct-header field fieldval))
-	    (:maildir  (mu4e~view-construct-header field fieldval))
-	    ((:flags :tags) (mu4e~view-construct-flags-tags-header field fieldval))
+	    (:subject    (mu4e~view-construct-header field fieldval))
+	    (:path       (mu4e~view-construct-header field fieldval))
+	    (:maildir    (mu4e~view-construct-header field fieldval))
+	    (:user-agent (mu4e~view-construct-header field fieldval))
+	    ((:flags :tags) (mu4e~view-construct-flags-tags-header
+			      field fieldval))
 
 	    ;; contact fields
 	    (:to       (mu4e~view-construct-contacts-header msg field))
@@ -237,7 +254,10 @@ found."
 		 (mu4e~view-custom-field msg field))))))
       mu4e-view-fields "")
     "\n"
-    (mu4e-message-body-text msg)))
+    (let ((body (mu4e-message-body-text msg mu4e-view-prefer-html)))
+      (when (fboundp 'add-face-text-property)
+        (add-face-text-property 0 (length body) 'mu4e-view-body-face t body))
+      body)))
 
 (defun mu4e~view-embedded-winbuf ()
   "Get a buffer (shown in a window) for the embedded message."
@@ -267,12 +287,12 @@ marking if it still had that."
 	    (if embedded
 	      (mu4e~view-embedded-winbuf)
 	      (get-buffer-create mu4e~view-buffer-name))))
-    ;; note: mu4e~view-mark-as-read will pseudo-recursively call mu4e-view again
-    ;; by triggering mu4e~view again as it marks the message as read
+    ;; note: mu4e~view-mark-as-read-maybe will pseudo-recursively call mu4e-view
+    ;; again by triggering mu4e~view again as it marks the message as read
     (with-current-buffer buf
       (switch-to-buffer buf)
       (setq mu4e~view-msg msg)
-      (when (or embedded (not (mu4e~view-mark-as-read msg)))
+      (when (or embedded (not (mu4e~view-mark-as-read-maybe msg)))
 	(let ((inhibit-read-only t))
 	  (erase-buffer)
 	  (mu4e~delete-all-overlays)
@@ -310,10 +330,15 @@ add text-properties to VAL."
   (let* ((info (cdr (assoc field
 		      (append mu4e-header-info mu4e-header-info-custom))))
 	  (key (plist-get info :name))
+	  (val (if val (propertize val 'field 'mu4e-header-field-value
+	                               'front-sticky '(field))))
 	  (help (plist-get info :help)))
     (if (and val (> (length val) 0))
     (with-temp-buffer
       (insert (propertize (concat key ":")
+		'field 'mu4e-header-field-key
+		'front-sticky '(field)
+		'keymap mu4e-view-header-field-keymap
 		'face 'mu4e-header-key-face
 		'help-echo help) " "
 	(if dont-propertize-val
@@ -331,6 +356,29 @@ add text-properties to VAL."
       (buffer-string))
     "")))
 
+(defun mu4e~view-header-field-fold ()
+  "Fold/unfold headers' value if there are more than one line."
+  (interactive)
+  (let ((name-pos (field-beginning))
+        (value-pos (1+ (field-end))))
+    (if (and name-pos value-pos
+             (eq (get-text-property name-pos 'field) 'mu4e-header-field-key))
+      (save-excursion
+        (let* ((folded))
+          (mapc (lambda (o)
+                  (when (overlay-get o 'mu4e~view-header-field-folded)
+                    (delete-overlay o)
+                    (setq folded t)))
+                (overlays-at value-pos))
+          (unless folded
+            (let* ((o (make-overlay value-pos (field-end value-pos)))
+                   (vals (split-string (field-string value-pos) "\n" t))
+                   (val (if (= (length vals) 1)
+                            (car vals)
+                          (concat (substring (car vals) 0 -3) "..."))))
+              (overlay-put o 'mu4e~view-header-field-folded t)
+              (overlay-put o 'display val))))))))
+
 (defun mu4e~view-compose-contact (&optional point)
   "Compose a message for the address at point."
   (interactive)
@@ -414,8 +462,9 @@ add text-properties to VAL."
   (let* ((parts (mu4e-message-field msg :parts))
 	 (verdicts
 	  (remove-if 'null
-		     (mapcar (lambda (part) (mu4e-message-part-field part :decryption))
-			     parts)))
+	    (mapcar (lambda (part)
+		      (mu4e-message-part-field part :decryption))
+	      parts)))
 	 (succeeded (remove-if (lambda (v) (eq v 'failed)) verdicts))
 	 (failed (remove-if (lambda (v) (eq v 'succeeded)) verdicts))
 	 (succ (when succeeded
@@ -522,7 +571,6 @@ FUNC should be a function taking two arguments:
   (dolist (part (mu4e-msg-field msg :parts))
     (funcall func msg part)))
 
-
 (defvar mu4e-view-mode-map nil
   "Keymap for \"*mu4e-view*\" buffers.")
 (unless mu4e-view-mode-map
@@ -536,7 +584,7 @@ FUNC should be a function taking two arguments:
 
       ;; note, 'z' is by-default bound to 'bury-buffer'
       ;; but that's not very useful in this case
-      (define-key map "z" 'mu4e~view-quit-buffer)
+      (define-key map "z" 'ignore)
 
       (define-key map "s" 'mu4e-headers-search)
       (define-key map "S" 'mu4e-view-search-edit)
@@ -558,6 +606,7 @@ FUNC should be a function taking two arguments:
 
       (define-key map "g" 'mu4e-view-go-to-url)
       (define-key map "k" 'mu4e-view-save-url)
+      (define-key map "f" 'mu4e-view-fetch-url)
 
       (define-key map "F" 'mu4e-compose-forward)
       (define-key map "R" 'mu4e-compose-reply)
@@ -568,6 +617,8 @@ FUNC should be a function taking two arguments:
       (define-key map "|" 'mu4e-view-pipe)
       (define-key map "a" 'mu4e-view-action)
 
+      (define-key map ";" 'mu4e-context-switch)
+      
       ;; toggle header settings
       (define-key map "O" 'mu4e-headers-change-sorting)
       (define-key map "P" 'mu4e-headers-toggle-threading)
@@ -594,9 +645,8 @@ FUNC should be a function taking two arguments:
       (define-key map (kbd "<M-down>") 'mu4e-view-headers-next)
       (define-key map (kbd "<M-up>") 'mu4e-view-headers-prev)
 
+      (define-key map (kbd "[") 'mu4e-view-headers-prev-unread)
       (define-key map (kbd "]") 'mu4e-view-headers-next-unread)
-      (define-key map (kbd "[")
-	(lambda() (interactive) (mu4e-view-headers-next-unread t)))
       
       ;; switching to view mode (if it's visible)
       (define-key map "y" 'mu4e-select-other-view)
@@ -631,7 +681,8 @@ FUNC should be a function taking two arguments:
 
       ;; misc
       (define-key map "w" 'visual-line-mode)
-      (define-key map "h" 'mu4e-view-toggle-hide-cited)
+      (define-key map "#" 'mu4e-view-toggle-hide-cited)
+      (define-key map "h" 'mu4e-view-toggle-html)
       (define-key map (kbd "M-q") 'mu4e-view-fill-long-lines)
 
       ;; next 3 only warn user when attempt in the message view
@@ -654,6 +705,8 @@ FUNC should be a function taking two arguments:
 	(define-key menumap [sepa0] '("--"))
 	(define-key menumap [wrap-lines]
 	  '("Toggle wrap lines" . visual-line-mode))
+	(define-key menumap [toggle-html]
+	  '("Toggle view-html" . mu4e-view-toggle-html))
 	(define-key menumap [hide-cited]
 	  '("Toggle hide cited" . mu4e-view-toggle-hide-cited))
 	(define-key menumap [raw-view]
@@ -668,6 +721,11 @@ FUNC should be a function taking two arguments:
 	  '("Open attachment" . mu4e-view-open-attachment))
 	(define-key menumap [extract-att]
 	  '("Extract attachment" . mu4e-view-save-attachment))
+
+	(define-key menumap [save-url]
+	  '("Save URL to kill-ring" . mu4e-view-save-url))
+	(define-key menumap [fetch-url]
+	  '("Fetch URL" . mu4e-view-fetch-url))
 	(define-key menumap [goto-url]
 	  '("Visit URL" . mu4e-view-go-to-url))
 
@@ -680,9 +738,10 @@ FUNC should be a function taking two arguments:
 	  '("Mark for move" . mu4e-view-mark-for-move))
 
 	(define-key menumap [sepa2] '("--"))
-	(define-key menumap [compose-new]  '("Compose new" . mu4e-compose-new))
+	(define-key menumap [resend]  '("Resend" . mu4e-compose-resend))
 	(define-key menumap [forward]  '("Forward" . mu4e-compose-forward))
 	(define-key menumap [reply]  '("Reply" . mu4e-compose-reply))
+	(define-key menumap [compose-new]  '("Compose new" . mu4e-compose-new))
 	(define-key menumap [sepa3] '("--"))
 
 	(define-key menumap [query-next]
@@ -695,12 +754,9 @@ FUNC should be a function taking two arguments:
 	  '("Search bookmark" . mu4e-headers-search-bookmark))
 	(define-key menumap [jump]
 	  '("Jump to maildir" . mu4e~headers-jump-to-maildir))
-	(define-key menumap [refresh]
-	  '("Refresh" . mu4e-headers-rerun-search))
 	(define-key menumap [search]
 	  '("Search" . mu4e-headers-search))
 
-
 	(define-key menumap [sepa4] '("--"))
 	(define-key menumap [next]  '("Next" . mu4e-view-headers-next))
 	(define-key menumap [previous]  '("Previous" . mu4e-view-headers-prev)))
@@ -726,24 +782,31 @@ FUNC should be a function taking two arguments:
   (make-local-variable 'mu4e~view-attach-map)
   (make-local-variable 'mu4e~view-cited-hidden)
 
-  (setq buffer-undo-list t) ;; don't record undo info
+  ;; make permanent too, so they'll survive changing the mode
+  (put 'mu4e~view-link-map 'permanent-local t)
+  (put 'mu4e~view-attach-map 'permanent-local t)
 
+  ;; show context in mode-string
+  (make-local-variable 'global-mode-string)
+  (add-to-list 'global-mode-string '(:eval (mu4e-context-label)))
+ 
+  (setq buffer-undo-list t);; don't record undo info
+     
   ;; autopair mode gives error when pressing RET
   ;; turn it off
   (when (boundp 'autopair-dont-activate)
     (setq autopair-dont-activate t)))
 
-(defun mu4e~view-mark-as-read (msg)
+(defun mu4e~view-mark-as-read-maybe (msg)
   "Clear the message MSG New/Unread status and set it to Seen.
 If the message is not New/Unread, do nothing. Evaluates to t if it
 triggers any changes, nil otherwise. If this function does any
 changes, it triggers a refresh."
-  (when msg
+  (when (and mu4e-view-auto-mark-as-read msg)
     (let ((flags (mu4e-message-field msg :flags))
 	   (msgid (mu4e-message-field msg :message-id))
 	   (docid (mu4e-message-field msg :docid)))
-      ;; attached (embedded) messages don't have docids; leave them alone
-      ;; is it a new message
+      ;; attached (embedded) messages don't have docids; leave them alone if it is a new message
       (when (and docid (or (member 'unread flags) (member 'new flags)))
 	;; mark /all/ messages with this message-id as read, so all copies of
 	;; this message will be marked as read.
@@ -795,8 +858,8 @@ If the url is mailto link, start writing an email to that address."
 
 (defvar mu4e~view-beginning-of-url-regexp
   "https?\\://\\|mailto:"
-  "Regexp that matches the beginning of http:/https:/mailto: URLs; match-string 1
-will contain the matched URL, if any.")
+  "Regexp that matches the beginning of http:/https:/mailto:
+URLs; match-string 1 will contain the matched URL, if any.")
 
 ;; this is fairly simplistic...
 (defun mu4e~view-make-urls-clickable ()
@@ -847,7 +910,8 @@ this view."
        (unless docid
 	 (mu4e-error "message without docid: action is not possible."))
        (with-current-buffer mu4e~view-headers-buffer
-	 (select-window (get-buffer-window))
+	 (when (get-buffer-window)
+	   (select-window (get-buffer-window)))
 	 (if (mu4e~headers-goto-docid docid)
 	   ,@body
 	   (mu4e-error "cannot find message in headers buffer."))))))
@@ -870,17 +934,29 @@ N (prefix argument), to the Nth previous header."
   (mu4e~view-in-headers-context
     (mu4e~headers-move (- (or n 1)))))
 
-(defun mu4e-view-headers-next-unread (&optional backwards)
+(defun mu4e~view-prev-or-next-unread (backwards)
   "Move point to the next or previous (when BACKWARDS is non-`nil')
 unread message header in the headers buffer connected with this
 message view. If this succeeds, return the new docid. Otherwise,
 return nil."
-  (interactive "P")
   (mu4e~view-in-headers-context 
-    (mu4e-headers-next-unread backwards))
+    (mu4e~headers-prev-or-next-unread backwards))
   (mu4e-select-other-view)
   (mu4e-headers-view-message))
 
+(defun mu4e-view-headers-prev-unread ()
+"Move point to the previous unread message header in the headers
+buffer connected with this message view. If this succeeds, return
+the new docid. Otherwise, return nil."
+  (interactive)
+  (mu4e~view-prev-or-next-unread t))
+
+(defun mu4e-view-headers-next-unread ()
+  "Move point to the next unread message header in the headers
+buffer connected with this message view. If this succeeds, return
+the new docid. Otherwise, return nil."
+  (interactive)
+  (mu4e~view-prev-or-next-unread nil))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -893,6 +969,12 @@ return nil."
     (mu4e-view-refresh)
     (mu4e~view-hide-cited)))
 
+(defun mu4e-view-toggle-html ()
+  "Toggle html-display of the message body (if any)."
+  (interactive)
+  (let ((mu4e-view-prefer-html (not mu4e~message-body-html))) 
+    (mu4e-view-refresh))) 
+
 (defun mu4e-view-refresh ()
   "Redisplay the current message."
   (interactive)
@@ -916,22 +998,31 @@ matching messages with that mark."
   (interactive)
   (mu4e~view-in-headers-context (mu4e-headers-mark-pattern)))
 
-(defun mu4e-view-mark-thread ()
-  "Ask user for a kind of mark (move, delete etc.), and apply it to
-all messages in the thread at point in the headers view."
+(defun mu4e-view-mark-thread (&optional markpair)
+  "Ask user for a kind of mark (move, delete etc.), and apply it
+to all messages in the thread at point in the headers view. The
+optional MARKPAIR can also be used to provide the mark
+selection."
   (interactive)
-  (mu4e~view-in-headers-context (mu4e-headers-mark-thread)))
+  (mu4e~view-in-headers-context
+   (if markpair (mu4e-headers-mark-thread nil markpair)
+       (call-interactively 'mu4e-headers-mark-thread))))
 
-(defun mu4e-view-mark-subthread ()
-  "Ask user for a kind of mark (move, delete etc.), and apply it to
-all messages in the subthread at point in the headers view."
+(defun mu4e-view-mark-subthread (&optional markpair)
+  "Ask user for a kind of mark (move, delete etc.), and apply it
+to all messages in the subthread at point in the headers view.
+The optional MARKPAIR can also be used to provide the mark
+selection."
   (interactive)
-  (mu4e~view-in-headers-context (mu4e-headers-mark-subthread)))
+  (mu4e~view-in-headers-context
+   (if markpair (mu4e-headers-mark-subthread markpair)
+     (mu4e-headers-mark-subthread))))
 
 (defun mu4e-view-search-narrow ()
   "Run `mu4e-headers-search-narrow' in the headers buffer."
   (interactive)
-  (mu4e~view-in-headers-context (call-interactively 'mu4e-headers-search-narrow)))
+  (mu4e~view-in-headers-context
+    (call-interactively 'mu4e-headers-search-narrow)))
 
 (defun mu4e-view-search-edit ()
   "Run `mu4e-headers-search-edit' in the headers buffer."
@@ -1058,7 +1149,8 @@ If ATTNUM is nil ask for the attachment number."
 	(and (file-exists-p fpath)
 	  (not (y-or-n-p (mu4e-format "Overwrite '%s'?" fpath))))))
     (mu4e~proc-extract
-      'save (mu4e-message-field msg :docid) index mu4e-decryption-policy fpath)))
+      'save (mu4e-message-field msg :docid)
+      index mu4e-decryption-policy fpath)))
 
 
 (defun mu4e-view-save-attachment-multi (&optional msg)
@@ -1083,14 +1175,17 @@ attachments, but as this is the default, you may not need it."
             (let* ((att (mu4e~view-get-attach msg num))
                    (fname  (plist-get att :name))
                    (index (plist-get att :index))
-                   (retry t))
+                   (retry t)
+		   fpath)
               (while retry
                 (setq fpath (expand-file-name (concat attachdir fname) path))
                 (setq retry
                       (and (file-exists-p fpath)
-                           (not (y-or-n-p (mu4e-format "Overwrite '%s'?" fpath))))))
+			(not (y-or-n-p
+			       (mu4e-format "Overwrite '%s'?" fpath))))))
               (mu4e~proc-extract
-               'save (mu4e-message-field msg :docid) index mu4e-decryption-policy fpath))))
+		'save (mu4e-message-field msg :docid)
+		index mu4e-decryption-policy fpath))))
       (dolist (num attachnums)
         (mu4e-view-save-attachment-single msg num)))))
 
@@ -1120,7 +1215,7 @@ If ATTNUM is nil ask for the attachment number."
       ;; send the docid as parameter (4th arg); we'll get this back from the
       ;; server, and use it to determine the parent message (ie., the current
       ;; message) when showing the embedded message/rfc822, and return to the
-      ;; current message when quiting that one.
+      ;; current message when quitting that one.
       (mu4e~view-temp-action docid index "mu4e" docid)
       ;; otherwise, open with the default program (handled in mu-server
       (mu4e~proc-extract 'open docid index mu4e-decryption-policy))))
@@ -1203,8 +1298,8 @@ attachments) in response to a (mu4e~proc-extract 'temp ... )."
     ((string= what "pipe")
       ;; 'param' will be the pipe command, path the infile for this
       (mu4e-process-file-through-pipe path param))
-    ;; if it's mu4e, it's some embedded message; 'param' may contain the docid of
-    ;; the parent message.
+    ;; if it's mu4e, it's some embedded message; 'param' may contain the docid
+    ;; of the parent message.
     ((string= what "mu4e")
       ;; remember the mapping path->docid, which maps the path of the embedded
       ;; message to the docid of its parent
@@ -1335,6 +1430,18 @@ to save a range of URLs."
       (kill-new url)
       (mu4e-message "Saved %s to the kill-ring" url))))
 
+(defun mu4e-view-fetch-url (&optional multi)
+  "Offer to fetch (download) urls(s). If MULTI (prefix-argument) is nil,
+download a single one, otherwise, offer to fetch a range of
+URLs. The urls are fetched to `mu4e-attachment-dir'."
+  (interactive "P")
+  (mu4e~view-handle-urls "URL to fetch" multi
+    (lambda (url)
+      (let ((target (concat (mu4e~get-attachment-dir url) "/"
+		      (file-name-nondirectory url))))
+	(url-copy-file url target)
+      (mu4e-message "Fetched %s -> %s" url target)))))
+
 (defun mu4e~view-handle-urls (prompt multi urlfunc)
   "If MULTI is nil, apply URLFUNC to a single uri, otherwise, apply
 it to a range of uris. PROMPT is the query to present to the user."
